@@ -20,7 +20,7 @@
  * @author     Daniel Convissor <danielc@php.net>
  * @copyright  1997-2005 The PHP Group
  * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
- * @version    CVS: $Id: odbc.php,v 1.78 2005/02/28 01:42:17 danielc Exp $
+ * @version    CVS: $Id: odbc.php,v 1.80 2007/01/12 03:11:17 aharvey Exp $
  * @link       http://pear.php.net/package/DB
  */
 
@@ -44,7 +44,7 @@ require_once 'DB/common.php';
  * @author     Daniel Convissor <danielc@php.net>
  * @copyright  1997-2005 The PHP Group
  * @license    http://www.php.net/license/3_0.txt  PHP License 3.0
- * @version    Release: @package_version@
+ * @version    Release: 1.7.11
  * @link       http://pear.php.net/package/DB
  */
 class DB_odbc extends DB_common
@@ -266,7 +266,7 @@ class DB_odbc extends DB_common
         }
         // Determine which queries that should return data, and which
         // should return an error code only.
-        if (DB::isManip($query)) {
+        if ($this->_checkManip($query)) {
             $this->affected = $result; // For affectedRows()
             return DB_OK;
         }
@@ -367,7 +367,7 @@ class DB_odbc extends DB_common
      */
     function freeResult($result)
     {
-        return @odbc_free_result($result);
+        return is_resource($result) ? odbc_free_result($result) : false;
     }
 
     // }}}
